/**
 * Stars!
 * 戦闘解決処理
 *
 * $Id: stars_battle.js,v 1.8 2006/02/12 11:07:30 shio Exp $
 */

// 戦闘解決処理クラス
function Battle() {
    this.damageList1 = null;
    this.damageList2 = null;

    /**
     * 艦隊戦を解決する。
     */
    this.conbatBattle = function(fleet1, fleet2) {
        this.damageList2 = this.fire(fleet1, fleet2);
        this.damageList1 = this.fire(fleet2, fleet1);
    }

    /**
     * 射撃を実施する。
     */
    this.fire = function(list1, list2) {
        var result = new Array();

        for (var i = 0; i < list1.length; i++) {
            var target = list1[i].makeTarget(list2);
            if (target && list1[i].fire(target) ) {
                result.push(target);
                list2 = list2.remove(list1[i].targetIx);
            }
        }

        return result;
    }

    /**
     * 戦闘結果を帝国艦隊に適用する。
     */
    this.applyBattleEmpireFleet = function(empire, fleet, list) {
        var msg = empire.name + "艦隊の被害は" + list.length + "<br/>";
        var result = fleet;
        for (var i = 0; i < list.length; i++) {
            var force = list[i];
            msg += force.refUnit.getName() + "壊滅<br/>";
            force.applyBattleEmpireFleet(empire);
            var nfleet = new Array();
            for (var j = 0; j < result.length; j++) {
                if (result[j].no != force.refUnit.no) {
                    nfleet.push(result[j]);
                }
            }
            result = nfleet;
        }
        if (result.length <= 0) {
            msg += empire.name + "の艦隊が壊滅しました。<br/>";
        }
        printConsole(empire.id, msg);
        return result;
    }

    /**
     * 戦闘結果を惑星守備隊に適用する。
     */
    this.applyBattlePlanetDfc= function(planet, list) {
        this.applyBattlePlanet(planet, list, "applyBattlePlanetDfc");
    }
    
    /**
    * 戦闘結果を惑星市民に適用する。
    */
    this.applyBattlePlanetDpl = function(planet, list) {
        this.applyBattlePlanet(planet, list, "applyBattlePlanetDpl");
    }
    
    /**
     * 惑星上の戦闘を解決する
     */
    this.applyBattlePlanet = function(planet, list, api) {
        var msgid = planet.ctl.id;
        var org_dfc = planet.dfc;
        var org_ecn = planet.ecn;
        var org_dpl = planet.dpl;
        for (var i = 0; i < list.length; i++) {
            var force = list[i];
            force[api](planet);
        }
        var dmg_dfc = org_dfc - planet.dfc;
        var dmg_ecn = org_ecn - planet.ecn;
        var dmg_dpl = org_dpl - planet.dpl;
        var msg = "";
        if (dmg_dfc > 0) {
            msg += "惑星" + planet.name + "防衛軍の被害は" + dmg_dfc + "<br/>";
        }
        if (dmg_ecn > 0) {
            msg += "惑星" + planet.name + "経済の被害は" + dmg_ecn + "<br/>";
        }
        if (dmg_dpl > 0) {
            msg += "惑星" + planet.name + "市民の被害は" + dmg_dpl + "<br/>";
            if (planet.dpl <= 0) {
                msg += "惑星" + planet.name + "が壊滅しました。<br/>";
                planet.dpl = 0;
                planet.ecn = 0;
                planet.ctl.unctl(planet);
                planet.ctl = null;
            }
        }
        if (msg != "") {
            printConsole(msgid, msg);
        }
    }

    /**
     * 戦闘結果を宇宙怪獣に適用する。
     */
    this.applyBattleUchuju = function(msgid, unit, list) {
        for (var i = 0; i < list.length; i++) {
            var force = list[i];
            force.applyBattleUchuju(unit);
        }
        var msg = "宇宙怪獣の被害は" + list.length + "<br/>";
        if (unit.force <= 0) {
            app.uchuju.removeUchuju(unit.id);
            msg += "宇宙怪獣を撃破しました。<br/>";
        }
        printConsole(msgid, msg);
    }

    /**
    * 戦闘結果を植民船団に適用する。
    */
    this.applyBattleSettler = function(empire, loc) {
        var msg = empire.name + "の植民船団(" + loc + ")が壊滅しました<br/>";
        empire.removeSettler(loc);
        
        printConsole(empire.id, msg);
    }
    
    this.makeSettlerForceList = function() {
        return new Array(new Force(0, 0));
    }

    this.getFleetForceList = function(fleet) {
        result = new Array();
        for (var i = 0; i < fleet.length; i++) {
            result.push(new Force(fleet[i].nvg, fleet[i].getExpMlt(), fleet[i]));
        }
        return result;
    }

    this.updateExpFleet = function(list) {
        for (var ix = 0; ix < list.length; ix++) {
            list[ix].refUnit.updateExpByBattle();
        }
    }
    
}

/**
 * 戦力管理クラス
 */
function Force(nvg, mlt, refUnit) {
    this.nvg = nvg;
    this.mlt = mlt;
    this.refUnit = (refUnit) ? refUnit : null;
    this.targetIx = 0;

    /**
     * どちらの戦力が強力かを比較する。
     */
    this.compare = function(another) {
        var result = 0;
        result = this.mlt - another.mlt;
        if (result != 0) {
            return result;
        }
        result = this.nvg - another.nvg;

        return result;
    }

    /**
     * 命中率を算出する。
     */
    this.rate = function(another) {
        var result = 30;

        var sub = this.mlt - another.mlt;
        if (sub == 0) {
            return result;
        } else if ( (sub == 1) || (sub == 2)) {
            return result + 10;
        } else if (sub > 2) {
            return result + (sub * 10) - 10;
        } else if (sub == -1) {
            return result - 10;
        } else {
            return result - 20;
        }

    }

    /**
     * 射撃を行なう
     */
    this.fire = function(target) {
        var rate = this.rate(target);
        var d = app.rollDice(100);
        return (d <= rate);
    }

    /**
     * 攻撃目標を決める
     */
    this.makeTarget = function(list) {
        var target = null;
        var maxRate = 0;
        this.targetIx = 0;
        for (var i = 0; i < list.length; i++) {
            var f = list[i];
            if (target == null) {
                target = f;
                continue;
            }
            var rate = this.rate(f);
            if (rate > maxRate) {
                maxRate = rate;
                target = f;
                this.targetIx = i;
            } else if (rate == maxRate) {
                if (this.compare(f) > 0) {
                    target = f;
                    this.targetIx = i;
                }
            }
        }

        return target;
    }

    this.key = function() {
        return this.nvg + "-" + this.mlt;
    }

    /**
     * 戦闘の結果を帝国艦隊に適用する.
     */
    this.applyBattleEmpireFleet = function(empire) {
        empire.removeFleet(this.refUnit);
    }

    /**
     * 戦闘結果を惑星守備隊に適用する。
     */
    this.applyBattlePlanetDfc = function(planet) {
        planet.dfc--;
        planet.ecn = (planet.ecn > 1) ? (planet.ecn - 1) : 0;
    }

    /**
     * 戦闘結果を惑星市民に適用する。
     */
    this.applyBattlePlanetDpl = function(planet) {
        planet.dpl--;
        planet.ecn = (planet.ecn > 2) ? (planet.ecn - 2) : 0;
    }

    /**
     * 戦闘結果を宇宙怪獣に適用する。
     */
    this.applyBattleUchuju = function(unit) {
        unit.force--;
    }
}

