/**
 * Stars!
 * 惑星クラス
 *
 * $Id: stars_planet.js,v 1.5 2006/02/13 14:37:13 shio Exp $
 */

// 惑星パラメータクラス
function Planet(id, loc, name, ctl, dpl, ecn, dfc, inc_p) {
    this.id = id;        // 識別番号(1 ～)
    this.loc = loc;        // 位置(r0c0形式)
    this.name = (name) ? name : "No." + id;        // 名称
    this.ctl = (ctl) ? ctl : null;    // 支配者(Empireオブジェクト)
    this.dpl = (dpl) ? dpl : 0;       // 開発レベル
    this.ecn = (ecn) ? ecn : 0;       // 経済レベル
    this.dfc = (dfc) ? dfc : 0;       // 防衛レベル
    this.inc_p = (inc_p) ? inc_p : 0; // 人口増加ポイント

    this.ecnLevel = function(mod) {
        mod = (mod) ? mod : 0;
        return this.ecn / (this.dpl + mod);
    }

    this.changeCtl = function(empire) {
        pre_ctl = this.ctl;
        if (pre_ctl) {
            pre_ctl.unctl(this);
        }
        empire.newctl(this);
        this.ctl = empire;
    }

    this.titleMsg = function() {
        var name = (this.name) ? this.name : "(No Name)";
        result = '';
        result += 'No.' + this.id + ' ' + name;
        result += ' 人-' + this.dpl;
        result += ' 経-' + this.ecn;
        result += ' 防-' + this.dfc;

        return result;
    }

    this.shogyoKenList = function() {
        var result = new Array();
        if (this.ctl == null) {
            return result;
        }
        for (var i = 0; i < this.ctl.ctlPlanetList.length; i++) {
            var p = this.ctl.ctlPlanetList[i];
            if (p.loc == this.loc) {
                continue;
            }
            var d = calDistance(this.loc, p.loc);
            if (d <= this.ctl.nvg) {
                result.push(p);
            }
        }

        return result;
    }

    // 航行可能な近隣惑星のリストを返す。
    this.navigatableList = function() {
        var result = new Array();
        if (this.ctl == null) {
            return result;
        }
        for (var lk in app.starList) {
            var p = app.starList[lk];
            if (p.loc == this.loc) {
                continue;
            }
            if (! this.ctl.isVisible(lk)) {
                continue;
            }
            var d = calDistance(this.loc, p.loc);
            if (d > this.ctl.nvg) {
                continue;
            }
            result.push(p);
        }

        return result;
    }

    this.shogyoKenTotalDpl = function() {
        var result = this.dpl;

        var list = this.shogyoKenList();
        for (var i = 0; i < list.length; i++) {
            var p = list[i];
            result += p.dpl;
        }

        return result;
    }

    /**
     * 惑星のコマンドを返す（Stars.getCmdByPlanet()のラッパ)
     */
    this.getCommand = function() {
        var cmd = app.getCmdByPlanetId(this.id);
        if (! (cmd) ) {
            cmd = new CmdNothing();
        }
        return cmd;
    }

    /**
     * 惑星のコマンド名を返す（Stars.getCmdByPlanet()のラッパ)
     */
    this.getCmdName = function() {
        var cmd = this.getCommand();
        if (cmd) {
            return cmd.name;
        } else {
            return "なし";
        }
    }

    this.setCmd = function(cmd) {
        app.setCmdByPlanetId(this.id, cmd);
    }

    this.makeForceList= function() {
        var list = new Array();
        for (var i = 0; i < this.dfc; i++) {
            list.push(new Force(0, this.ctl.mlt, this));
        }
        return list;
    }

    this.makeDplForceList = function() {
        var list = new Array();
        for (var i = 0; i < this.dpl; i++) {
            list.push(new Force(0, 0, this));
        }
        return list;
    }

    // 最大経済力を返す。
    this.calMaxEcn = function() {
        var shoken = this.shogyoKenTotalDpl();
        var dplEcn = this.dpl * 2;
        return Math.min(shoken, dplEcn, 9);
    }

    return this;
}
